local LG = LibStub("AceLocale-3.0"):GetLocale("Grid2")
local L = LibStub("AceLocale-3.0"):GetLocale("Grid2Options")

local Grid2Options = {
	options = {
		name = "Grid2 Continued",
		type = "group",
		handler = Grid2,
		args = {
			["Tabs"] = {
				order = 10,
				type = "group",
				name = L["General Settings"],
				desc = L["General Settings"],
				childGroups = "tab",
				args = {}, -- Settings tabs here		
			},
		},
	},
	typeMakeOptions = {},
	optionParams = {},	
}

local TABS_ORDER_DISPLAY = 10
local SECT_ORDER_DISPLAY = 10

function Grid2Options:AddModuleOptions(TabName, SectionName, extraOptions)
	local Tabs  = Grid2Options.options.args["Tabs"]
	local CurTab= Tabs.args[TabName]
	if (not CurTab) and (SectionName or (not extraOptions.args)) then
		CurTab= { type = "group", order = TABS_ORDER_DISPLAY,	name = L[TabName], args = {} }
		TABS_ORDER_DISPLAY= TABS_ORDER_DISPLAY + 1
		Tabs.args[TabName]= CurTab
	end
	if SectionName then
		local CurSec= CurTab.args[SectionName]
		if CurSec then
			for key,value in pairs(extraOptions) do
				CurSec.args[key]= value
			end
		else
			if extraOptions.args  then
				CurTab.args[SectionName]= extraOptions
			else	
				CurTab.args[SectionName]= { type = "group", inline= true, order = SECT_ORDER_DISPLAY, 
											name = L[SectionName],	desc = L["Options for %s."]:format(L[SectionName]),
											args = extraOptions,	}
				SECT_ORDER_DISPLAY = SECT_ORDER_DISPLAY + 1
			end
		end			
	else
		if extraOptions.args then
			Tabs.args[TabName]= extraOptions
		else
			for key,value in pairs(extraOptions) do
				CurTab.args[key]= value
			end
		end
	end
	
end

function Grid2Options:AddElement(elementType, element, extraOptions)
	--Elementtype: a string representing the options
	--Element: The element itself
	--ExtraOptions: The aceconfig structure
	--
	--Adds options for this element to the main menu.
	--Will create a menu of type elementType if it doesn't already exist.
	--That in turn must be a group, with elements matching 'element'
	--
	--The OO here is a bit laboured :(
	
	extraOptions = extraOptions or element.extraOptions
	element.extraOptions = nil
	if not extraOptions then return end

	local group = self.options.args[elementType]
	if not group then
		group = {
			type = "group",
			name = L[elementType] or elementType,
			desc = L["Options for %s."]:format(elementType),
			args = {},
		}
		self.options.args[elementType] = group
	end
	local options = {}
	group.args[element.name] = {
		type = "group",
		order = 100,
		name = L[element.name] or element.name,
		desc = L["Options for %s."]:format(element.name),
		args = options,
	}
	for name, option in pairs(extraOptions) do
		options[name] = option
	end
end

function Grid2Options:DeleteElement(elementType, elementKey)
	local group = self.options.args[elementType]
	if group then
		group.args[elementKey] = nil
	end
end

function Grid2Options:DeleteElementSubType(elementType, subType, elementKey)
	local group = self.options.args[elementType]
	if group then
		local subGroup = group.args[subType]
		if (subGroup) then
			subGroup.args[elementKey] = nil
		end
	end	
end

-- Adds meta options for the list of elements from AddElement
-- Order < 100 is reserved for Grid elements
-- If reset is true then discard the old options
function Grid2Options:AddElementGroup(type, extraOptions, order, reset)
	if not extraOptions then return end

	local group = self.options.args[type]
	if (reset or not group) then
		group = {
			type = "group",
			order = order,
			name = L[type] or type,
			desc = L["Options for %s."]:format(type),
			args = {},
		}
		self.options.args[type] = group
	end
	local options = group.args
	for name, option in pairs(extraOptions) do
		options[name] = option
	end
end

function Grid2Options:AddElementSubTypeGroup(type, subType, subTypeDescription, subTypeOptions, reset)
	local group = self.options.args[type]
	if (not group) then
		group = {
			type = "group",
			name = L[type] or type,
			desc = L["Options for %s."]:format(type),
			args = {},
		}
		self.options.args[type] = group
	end

	local subGroup = group.args[subType]
	local options = {}
	if (reset or not subGroup) then
		subGroup = {
			type = "group",
			name = L[subTypeDescription] or subTypeDescription or L[subType] or subType,
			order = 10,
			desc = L["Options for %s."]:format(subType),
			args = options,
		}
		group.args[subType] = subGroup
	end
	if (subTypeOptions) then
		for name, option in pairs(subTypeOptions) do
			options[name] = option
		end
	end
	return subGroup
end


function Grid2Options:AddElementSubType(elementType, subType, element, extraOptions)
	extraOptions = extraOptions or element.extraOptions
	element.extraOptions = nil
	if not extraOptions then return end

	local group = self.options.args[elementType]
	if not group then
		group = {
			type = "group",
			name = L[elementType] or elementType,
			desc = L["Options for %s."]:format(elementType),
			args = {},
		}
		self.options.args[elementType] = group
	end

	local subGroup = group.args[subType]
	if (not subGroup) then
		subGroup = self:AddElementSubTypeGroup(elementType, subType)
	end

	local name= Grid2Options.LocalizeStatus(element.name, true)
	
	-- Calculate order: Magic,Curse,Poison and Disease debuffs first
	local order= 10
	if not element.dbx.subType then order= 20 end
	
	local options = {}
	subGroup.args[element.name] = {
		type = "group",
		name = name,
		order = order,
		desc = L["Options for %s."]:format(element.name),
		args = options,
	}
	for name, option in pairs(extraOptions) do
		options[name] = option
	end
end

function Grid2Options:DeleteElementSubType(elementType, subType, elementKey)
	local group = self.options.args[elementType]
	if not group then
		return
	end

	local subGroup = group.args[subType]
	if (not subGroup) then
		return
	end
	subGroup.args[elementKey] = nil
end

function Grid2Options:AddOptionHandler(typeKey, funcMakeOptions, optionParams)
	Grid2Options.typeMakeOptions[typeKey] = funcMakeOptions
	Grid2Options.optionParams[typeKey] = optionParams
end

function Grid2Options:GetOptionHandler(typeKey)
	return Grid2Options.typeMakeOptions[typeKey], Grid2Options.optionParams[typeKey]
end

function Grid2Options:GetLayouts( meta )
    local list= { }
	for name, layout in pairs(Grid2Layout.layoutSettings) do
      if layout.meta[meta] == true then
	     list[name]= L[name]
	  end
	end
	return list
end

---
---
---

function Grid2Options:Initialize()
	self = self or Grid2Options

	-- Create general options tabs
	Grid2Options:MakeFrameOptions()
	Grid2Options:MakeLayoutOptions()
	Grid2Options:MakeBlinkOptions()
    Grid2Options:MakeProfileOptions()
	Grid2Options:MakeDebugOptions()
	Grid2Options:MakeAboutOptions()
	-- Create indicators, statuses options 
	Grid2Options:MakeOptions()
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Grid2", self.options)

	local ACD3 = LibStub("AceConfigDialog-3.0")
	for key, value in pairs(self.options.args) do
		 if (key ~= "Tabs") then
			 ACD3:AddToBlizOptions("Grid2", value.name, "Grid2", key)
		 end
	end

	self.Initialize = nil
end

function Grid2Options:MakeProfileOptions(reset)
	
	local profileOptions = LibStub('AceDBOptions-3.0'):GetOptionsTable(Grid2.db, true)
    local LibDualSpec = LibStub('LibDualSpec-1.0')
	if LibDualSpec then
		LibDualSpec:EnhanceOptions(profileOptions, Grid2.db)
	end
	Grid2Options:AddModuleOptions("Profiles", nil, profileOptions )
end

function Grid2Options:MakeAboutOptions(reset)
	Grid2Options:AddModuleOptions("About", nil, {
        p1 = { order = 10, type = "description", name = "\n\n"..Grid2.versionstring},
		p2 = { order = 11, type = "description", name = "\nGrid2 is a party and raid unit frame addon." },
        p3 = { order = 12, type = "description", name = "Grid2 displays health and all relevant information about the raid members in a more comprehensible manner.\n" },
	})
end

-- Called from Grid2 if profile changed
function Grid2Options:MakeOptions()
    Grid2Options:MakeStatusOptions(true)	
	Grid2Options:MakeIndicatorOptions(true)
end

function Grid2Options:OnChatCommand(input)
	if (LibStub("AceConfigDialog-3.0").OpenFrames["Grid2"]) then
		LibStub("AceConfigDialog-3.0"):Close("Grid2")
	else
		LibStub("AceConfigDialog-3.0"):Open("Grid2")
	end
end

function Grid2Options:GetValidatedName(name)
	name = name:gsub("[\"%.]", "")
	name = name:gsub(" ", "-")
	return name
end

_G.Grid2Options = Grid2Options

