local L = LibStub("AceLocale-3.0"):GetLocale("Grid2Options")
local media = LibStub("LibSharedMedia-3.0", true)

local Grid2Options = Grid2Options

-- List of indicator types that can be created
local newIndicatorTypes = {}

local pointMap = {
	TOPLEFT = "1",
	LEFT = "2",
	BOTTOMLEFT = "3",
	TOP = "4",
	CENTER = "5",
	BOTTOM = "6",
	TOPRIGHT = "7",
	RIGHT = "8",
	BOTTOMRIGHT = "9",
	["1"] = "TOPLEFT",
	["2"] = "LEFT",
	["3"] = "BOTTOMLEFT",
	["4"] = "TOP",
	["5"] = "CENTER",
	["6"] = "BOTTOM",
	["7"] = "TOPRIGHT",
	["8"] = "RIGHT",
	["9"] = "BOTTOMRIGHT",
}

local pointValueList = {
	["1"] = L["TOPLEFT"],
	["2"] = L["LEFT"],
	["3"] = L["BOTTOMLEFT"],
	["4"] = L["TOP"],
	["5"] = L["CENTER"],
	["6"] = L["BOTTOM"],
	["7"] = L["TOPRIGHT"],
	["8"] = L["RIGHT"],
	["9"] = L["BOTTOMRIGHT"],
}

function Grid2Options.GetNewIndicatorTypes()
	return newIndicatorTypes
end

function Grid2Options:UpdateIndicator(indicator)
	local baseKey = indicator.name
	if (indicator.UpdateDB) then
		indicator:UpdateDB(Grid2.indicators[baseKey].dbx)
	end
end

function Grid2Options:GetNewStatusPriority(indicator)
	local priority = 99
	
	for key, value in pairs(indicator.priorities) do
		priority = math.max(priority, value + 1)
	end

	return priority
end

function Grid2Options:RegisterIndicatorStatus(indicator, status, newPriority)
	local baseKey = indicator.name
	local statusKey = status.name
	local priority = newPriority or Grid2Options:GetNewStatusPriority(indicator)
	
	if not Grid2.db.profile.statusMap[baseKey] then
		Grid2.db.profile.statusMap[baseKey]= {}
	end
    Grid2.db.profile.statusMap[baseKey][statusKey]= priority
	indicator:RegisterStatus(status, priority)
end

function Grid2Options:UnregisterIndicatorStatus(indicator, status)
	local baseKey = indicator.name
	local statusKey = status.name
	
    Grid2.db.profile.statusMap[baseKey][statusKey]= nil
	indicator:UnregisterStatus(status)
end


-- Wrapper for indicator:SetStatusPriority that sets priority in setup as well
function Grid2Options:SetStatusPriority(indicator, status, priority)
	local baseKey = indicator.name
	local statusKey = status.name

    Grid2.db.profile.statusMap[baseKey][statusKey]= priority

	indicator:SetStatusPriority(status, priority)
end


function Grid2Options.GetIndicatorStatus(info, statusKey)
	local indicator = info.arg
	statusKey = statusKey or info[# info]

	for key, status in Grid2:IterateStatuses() do
		if (key == statusKey) then
			return status.indicators[indicator]
		end
	end

	return false
end

function GetParentOption(info, objectKey)
	local settings= info.options.args.indicators.args[objectKey].args
	return settings.statusesCurrent or settings.statuses.args.statusesCurrent
end

function Grid2Options.SetIndicatorStatusCurrent(info, value)
	local indicator = info.arg
	local statusKey = info[# info]

	for key, status in Grid2:IterateStatuses() do
		if (key == statusKey) then
			if (value) then
				Grid2Options:RegisterIndicatorStatus(indicator, status)
			else
				Grid2Options:UnregisterIndicatorStatus(indicator, status)
			end
			Grid2Frame:WithAllFrames(function (f) indicator:Layout(f) end)
			Grid2Frame:ResetAllFrames()
			Grid2Frame:UpdateAllFrames()

			local parentOption = GetParentOption(info, indicator.name)
			wipe(parentOption.args)
			Grid2Options:AddIndicatorCurrentStatusOptions(indicator, parentOption.args)
		end
	end
end
--/dump Grid2Options.options.Grid2.args.indicator.args.alpha

function Grid2Options.SetIndicatorStatus(info, statusKey, value)
	local indicator = info.arg

	for key, status in Grid2:IterateStatuses() do
		if (key == statusKey) then
			if (value) then
				Grid2Options:RegisterIndicatorStatus(indicator, status, 99)
			else
				Grid2Options:UnregisterIndicatorStatus(indicator, status)
			end
			Grid2Frame:WithAllFrames(function (f) indicator:Layout(f) end)
			Grid2Frame:ResetAllFrames()
			Grid2Frame:UpdateAllFrames()

			local parentOption = GetParentOption(info, indicator.name)
			wipe(parentOption.args)
			Grid2Options:AddIndicatorCurrentStatusOptions(indicator, parentOption.args)
		end
	end
end

local function StatusShiftUp(info, indicator, lowerStatus)
	for index, status in ipairs(indicator.statuses) do
		if (lowerStatus == status) then
			local newIndex = index - 1
			if (newIndex > 0) then
				local higherStatus = indicator.statuses[newIndex]
				local higherPriority = indicator:GetStatusPriority(higherStatus)
				local lowerPriority = indicator:GetStatusPriority(lowerStatus)
--print("StatusShiftUp", lowerPriority, higherPriority, lowerStatus.name, higherStatus.name)
				if (higherPriority == lowerPriority) then
					higherPriority = higherPriority + 1
				end
-- print("StatusShiftUp", lowerPriority, higherPriority, lowerStatus.name, higherStatus.name)
				Grid2Options:SetStatusPriority(indicator, higherStatus, lowerPriority)
				Grid2Options:SetStatusPriority(indicator, lowerStatus, higherPriority)

				local parentOption = GetParentOption(info, indicator.name)
				wipe(parentOption.args)
				Grid2Options:AddIndicatorCurrentStatusOptions(indicator, parentOption.args)
			end
			break
		end
	end
end

local function StatusShiftDown(info, indicator, higherStatus)
	for index, status in ipairs(indicator.statuses) do
		if (higherStatus == status) then
			local newIndex = index + 1
			if (newIndex <= # indicator.statuses) then
				local lowerStatus = indicator.statuses[newIndex]
				local higherPriority = indicator:GetStatusPriority(higherStatus)
				local lowerPriority = indicator:GetStatusPriority(lowerStatus)
				if (higherPriority == lowerPriority) then
					lowerPriority = lowerPriority - 1
				end
-- print("StatusShiftDown", lowerPriority, higherPriority, lowerStatus.name, higherStatus.name)
				Grid2Options:SetStatusPriority(indicator, higherStatus, lowerPriority)
				Grid2Options:SetStatusPriority(indicator, lowerStatus, higherPriority)

				local parentOption = GetParentOption(info, indicator.name)
				wipe(parentOption.args)
				Grid2Options:AddIndicatorCurrentStatusOptions(indicator, parentOption.args)
			end
			break
		end
	end
end

function Grid2Options:AddIndicatorCurrentStatusOptions(indicator, options)
	if (indicator.statuses) then
		for index, status in ipairs(indicator.statuses) do
			local statusKey = status.name
			local order = 5 * index
			local passValue = {indicator = indicator, status = status}
			options[statusKey] = {
				type = "toggle",
				order = order,
				--width= "double",
				name =  Grid2Options.LocalizeStatus(statusKey),
				desc = L["Select statuses to display with the indicator"],
				get = Grid2Options.GetIndicatorStatus,
				set = Grid2Options.SetIndicatorStatusCurrent,
				arg = indicator,
			}
			options[statusKey .. "U"] = {
			    type = "execute",
				order = order + 1,
				width = "half",
			    name = L["+"],
			    desc = L["Move the status higher in priority"],
				icon = "Interface\\Buttons\\UI-MicroButton-Spellbook-Up",
			    func = function (info)
			    	StatusShiftUp(info, indicator, status)
				end,
				arg = indicator,
			}
			options[statusKey .. "D"] = {
			    type = "execute",
				order = order + 2,
				width = "half",
			    name = L["-"],
			    desc = L["Move the status lower in priority"],
				icon = "Interface\\Buttons\\UI-MicroButton-Spellbook-Down",
			    func = function (info)
			    	StatusShiftDown(info, indicator, status)
				end,
				arg = indicator,
			}
			options[statusKey .. "S"] = {
				type = "header",
				order = order + 3,
				name = "",
			}
		end
	end
end

function Grid2Options:AddIndicatorStatusOptions(indicator, options)
	options.statusesCurrent = {
		type = "group",
		order = 100,
		inline = true,
		name = L["Current Statuses"],
		desc = L["Current statuses in order of priority"],
		args = {},
	}
	Grid2Options:AddIndicatorCurrentStatusOptions(indicator, options.statusesCurrent.args)

	options.statusesAvailable = {
	    type = "multiselect",
		order = 200,
		name = L["Available Statuses"],
		desc = L["Available statuses you may add"],
		values = function (info)
			local statusAvailable = Grid2Options:GetAvailableStatusValues(indicator)
			return statusAvailable
		end,
		get = Grid2Options.GetIndicatorStatus,
		set = Grid2Options.SetIndicatorStatus,
		arg = indicator,
	}
end


local function DeleteIndicator(indicator)

	-- Disable indicator on visible unit frames 
	Grid2Frame:WithAllFrames(function (f) indicator:Disable(f) end)

	-- Remove indicator and its stasuses from runtime	
	Grid2:UnregisterIndicator(indicator) 

	-- remove indicator from database	
	local baseKey = indicator.name
	Grid2:DbSetIndicator(baseKey,nil)
 	Grid2Options:DeleteElement("indicators", baseKey)

	-- remove asociated color indicator if exists
	baseKey = baseKey .. '-color'
	if Grid2:DbGetIndicator(baseKey) then
		Grid2:DbSetIndicator(baseKey, nil)
	end	
	Grid2Options:DeleteElement("indicators", baseKey)
	
	Grid2Frame:ResetAllFrames()
	Grid2Frame:UpdateAllFrames()
end


function Grid2Options:MakeIndicatorSizeOptions(indicator, options, optionParams)
	options = options or {}
	local baseKey = indicator.name

	local name = L["Border Size"]
	local desc = L["Adjust the border size of the indicator."]
	options.size = {
		type = "range",
		order = 10,
		name = L["Size"],
		desc = L["Adjust the size of the indicator."],
		min = 5,
		max = 50,
		step = 1,
		get = function ()
			return indicator.dbx.size
		end,
		set = function (_, v)
			indicator.dbx.size = v
			Grid2.db.profile.indicators[baseKey].size = v	-- ToDo: handle nilling out if ancestor matches v
			Grid2Frame:WithAllFrames(function (f)
				indicator:SetIndicatorSize(f, v)
				indicator:Layout(f)
			end)
		end,
	}

	return options
end


function Grid2Options:MakeIndicatorBorderSizeOptions(indicator, options, optionParams)
	options = options or {}
	local baseKey = indicator.name

	local name = L["Border"]
	local desc = L["Adjust the border size of the indicator."]
	options.borderSize = {
		type = "range",
		order = 20,
		name = name,
		desc = desc,
		min = 0,
		max = 20,
		step = 1,
		get = function ()
			return indicator.dbx.borderSize or 0
		end,
		set = function (_, v)
			if (v == 0) then
				v = nil
			end
			indicator.dbx.borderSize = v
			Grid2.db.profile.indicators[baseKey].borderSize = v	-- ToDo: handle nilling out if ancestor matches v
			Grid2Frame:WithAllFrames(function (f)
				indicator:SetBorderSize(f, v)
				indicator:Layout(f)
			end)
		end,
	}

	return options
end


function Grid2Options:MakeIndicatorBorderOptions(indicator, options, optionParams)
	options = options or {}
	optionParams = optionParams or {}
	optionParams.color1 = L["Border"]
	optionParams.colorDesc1 = L["Adjust border color and alpha."]
	optionParams.typeKey = "indicators"

	Grid2Options:MakeIndicatorColorOptions(indicator, options, optionParams)
	Grid2Options:MakeIndicatorBorderSizeOptions(indicator, options, optionParams)
	
	return options
end


function Grid2Options:AddIndicatorDeleteOptions(indicator, options)
	options.deleteHeader = {
			type = "header",
			order = 151,
			name = "",
		}
	options.delete = {
	    type = "execute",
		order = 152,
	    name = L["Delete"],
	    func = function (info)
		  DeleteIndicator(indicator)
		end,
		arg = indicator,
	}
end


function Grid2Options.GetIndicatorColor(info)
	local passValue = info.arg
	local indicator = passValue.indicator
	local colorKey = "color"

	local colorIndex = passValue.colorIndex
	colorKey = colorKey .. colorIndex

	local c = indicator.dbx[colorKey]
	if (c) then
		return c.r, c.g, c.b, c.a
	else
		return 0, 0, 0, 0
	end
end

function Grid2Options.SetIndicatorColor(info, r, g, b, a)
	local passValue = info.arg
	local indicator = passValue.indicator
	local typeKey = passValue.typeKey
	local dbx = Grid2.db.profile[typeKey][indicator.name]           
	local colorKey = "color"

	local colorIndex = passValue.colorIndex
	colorKey = colorKey .. colorIndex

	local c = indicator.dbx[colorKey]
	if (not c) then
		c = {}
		indicator.dbx[colorKey] = c
	end
	c.r, c.g, c.b, c.a = r, g, b, a

	c = dbx[colorKey]
	if (not c) then
		c = {}
		dbx[colorKey] = c
	end
	c.r, c.g, c.b, c.a = r, g, b, a

	Grid2Frame:ResetAllFrames()
	Grid2Frame:UpdateAllFrames()
end

function Grid2Options:MakeIndicatorColorOptions(indicator, options, optionParams)
	options = options or {}

	local colorCount = indicator.dbx.colorCount or 1
--print("MakeIndicatorColorOption", indicator.name, colorCount)
	local name = L["Color"]
	local desc = L["Color for %s."]:format(indicator.name)
	local typeKey = optionParams and optionParams.typeKey or "indicators"
	for i = 1, colorCount, 1 do
		local colorKey = "color" .. i
		if (optionParams and optionParams[colorKey]) then
			name = optionParams[colorKey]
		elseif (colorCount > 1) then
			name = L["Color %d"]:format(i)
		end

		local colorDescKey = "colorDesc" .. i
		if (optionParams and optionParams[colorDescKey]) then
			desc = optionParams[colorDescKey]
		elseif (colorCount > 1) then
			desc = name
		end

		options[colorKey] = {
			type = "color",
			order = (20 + i),
			name = name,
			desc = desc,
			get = Grid2Options.GetIndicatorColor,
			set = Grid2Options.SetIndicatorColor,
			hasAlpha = true,
			arg = {indicator = indicator, colorIndex = i, typeKey = typeKey},
		}
	end

	return options
end

Grid2Options.typeMorphValues = {
	icon = {icon = L["icon"], square = L["square"], text = L["text"]},
	square = {square = L["square"], text = L["text"],icon = L["icon"]},
	text = {square = L["square"], text = L["text"],icon = L["icon"]},
}

function Grid2Options.GetIndicatorTypeValues(info)
	local indicator = info.arg
	local typeKey = indicator.dbx.type
	local typeMorphValues = Grid2Options.typeMorphValues
	
	if (not typeMorphValues[typeKey]) then
		typeMorphValues[typeKey] = {}
		typeMorphValues[typeKey][typeKey] = L[typeKey]
	end
	
	return Grid2Options.typeMorphValues[typeKey]
end

function Grid2Options.GetIndicatorType(info)
	local indicator = info.arg
	
	return indicator.dbx.type
end

local defaultFont = "Friz Quadrata TT"

Grid2Options.typeDefaultValues = {
	icon = {size = 16, fontSize = 8,},
	square = {size = 5,},
	text = { duration = true, stack= false, textlength = 12, fontSize = 8, font = defaultFont,},
}

local function RegisterIndicatorStatusesFromDatabase(indicator)
	if indicator then
		local map= Grid2:DbGetValue("statusMap", indicator.name)
		if map then
			for statusKey, priority in pairs(map) do
				local status = Grid2.statuses[statusKey]
				if (status and tonumber(priority)) then
					indicator:RegisterStatus(status, priority)
				end
			end	
		end	
	end
end	

function Grid2Options.SetIndicatorType(info, value)
	local indicator = info.arg
	local baseKey = indicator.name
	local colorKey = baseKey.."-color"
	local oldType = indicator.dbx.type
	local dbx = Grid2.db.profile.indicators[baseKey]

	if  indicator.dbx.type == value then return end
	
	-- Set new fields width defaults values
	dbx.type = value
	for k, v in pairs(Grid2Options.typeDefaultValues[value]) do
		if (not dbx[k]) then
			indicator.dbx[k] = v
			dbx[k] = v
		end
	end
	-- Remove old indicator
	Grid2Frame:WithAllFrames(function (f) indicator:Disable(f) end)
	Grid2:UnregisterIndicator(indicator)
	-- Create new indicator
	local setupFunc = Grid2.setupFunc[dbx.type]
	local newIndicator = setupFunc(baseKey, dbx)
	-- remove incompatible statuses from database
	local map= Grid2:DbGetValue("statusMap", baseKey)
	for statusKey, priority in pairs(map) do
		local status = Grid2.statuses[statusKey]
    	if (not status) or (not Grid2:IsCompatiblePair(newIndicator, status)) then
			map[statusKey]= nil
			print("Incompatible status",statusKey)
		end
	end
	-- Register indicator statuses from database
	RegisterIndicatorStatusesFromDatabase(newIndicator)
	RegisterIndicatorStatusesFromDatabase(newIndicator.sideKick)
	-- Recreate indicators in frame units 
	Grid2Frame:WithAllFrames(function (f)
		newIndicator:Create(f)
		newIndicator:Layout(f)
	end)
	-- Delete or Create associated text-color indicator in database
	if oldType=="text" then
		Grid2:DbSetIndicator(colorKey, nil)
	elseif value=="text" then
		Grid2:DbSetIndicator( colorKey , { type="text-color" })
	end
	-- Delete old indicator options
	Grid2Options:DeleteElement("indicators", baseKey)
	Grid2Options:DeleteElement("indicators", baseKey.."-color")
	-- Create new indicator options
	local funcMakeOptions = Grid2Options.typeMakeOptions[dbx.type]
	if (funcMakeOptions) then
		funcMakeOptions(newIndicator)
	end

	Grid2Frame:UpdateAllFrames()
end

function Grid2Options:MakeIndicatorTypeOptions(indicator, options, optionParams)
	local baseKey = indicator.name
	options.type = {
	    type = 'select',
		order = 3,
		name = L["Type"],
		desc = L["Type of indicator"],
	    values = Grid2Options.GetIndicatorTypeValues,
	    get = Grid2Options.GetIndicatorType,
	    set = Grid2Options.SetIndicatorType,
		arg = indicator,
	}

end

function Grid2Options:AddIndicatorLocationOptions(indicator, options)
	local baseKey   = indicator.name
	local location  = indicator.dbx.location
	local locatidb  = Grid2.db.profile.indicators[baseKey].location
	options.locationSeparator1 = {
			type = "header",
			order = 4,
			name = L["Location"],
	}
	options.relPoint = {
		    type = 'select',
			order = 5,
			name = L["Location"],
			desc = L["Align my align point relative to"],
		    values = pointValueList,
			get = function() return pointMap[location.relPoint] end,
			set = function(_, v)
					location.relPoint= pointMap[v]
					locatidb.relPoint= location.relPoint
					indicator.anchorRel = location.relPoint
					location.point= location.relPoint
					locatidb.point= location.relPoint
					indicator.anchor= location.relPoint
					Grid2Frame:WithAllFrames(function (f) indicator:Layout(f) end)
			end,
		}
	options.point = {
		    type = 'select',
			order = 6,
			name = L["Align Point"],
			desc = L["Align this point on the indicator"],
		    values = pointValueList,
			get = function() return pointMap[location.point] end,
			set = function(_, v)
					location.point = pointMap[v] 
					locatidb.point = location.point
					indicator.anchor = location.point
					Grid2Frame:WithAllFrames(function (f) indicator:Layout(f) end)
			end,
		}
	options.x = {
			type = "range",
			order = 7,
			name = L["X Offset"],
			desc = L["X - Horizontal Offset"],
			min = -50, max = 50, step = 1, bigStep = 1,
			get = function() return location.x end,
			set = function(_, v)
					location.x = v 
					locatidb.x = v
					indicator.offsetx = v
					Grid2Frame:WithAllFrames(function (f) indicator:Layout(f) end)
			end,
		}
	options.y = {
			type = "range",
			order = 8,
			name = L["Y Offset"],
			desc = L["Y - Vertical Offset"],
			min = -50, max = 50, step = 1, bigStep = 1,
			get = function() return location.y end,
			set = function(_, v)
					location.y = v
					locatidb.y = v	
			        indicator.offsety = v
					Grid2Frame:WithAllFrames(function (f) indicator:Layout(f) end)
			end,
		}
	options.locationSeparator2 = {
			type = "header",
			order = 9,
			name = L["Appearance"],
		}		

end

local function MakeTextIndicatorOptions(indicator)
	local baseKey = indicator.name
	local statuses= {}
	local options = {
		textlength = {
			type = "range",
			order = 10,
			name = L["Text Length"],
			desc = L["Maximum number of characters to show."],
			min = 0,
			max = 20,
			step = 1,
			get = function () return indicator.dbx.textlength end,
			set = function (_, v)
				indicator.dbx.textlength = v
				Grid2.db.profile.indicators[baseKey].textlength = v
				Grid2Frame:UpdateAllFrames()
			end,
		},
		fontsize = {
			type = "range",
			order = 20,
			name = L["Font Size"],
			desc = L["Adjust the font size."],
			min = 6,
			max = 24,
			step = 1,
			get = function ()
				return indicator.dbx.fontSize
			end,
			set = function (_, v)
				indicator.dbx.fontSize = v
				Grid2.db.profile.indicators[baseKey].fontSize = v
				local font = media and media:Fetch('font', indicator.dbx.font) or STANDARD_TEXT_FONT
				Grid2Frame:WithAllFrames(function (f) indicator:SetTextFont(f, font, v) end)
			end,
		},
		duration = {
			type = "toggle",
			name = L["Show duration"],
			desc = L["Show the time remaining."],
			order = 80,
			tristate = true,
			get = function ()
				return indicator.dbx.duration
			end,
			set = function (_, v)
				indicator.dbx.duration = v
				Grid2.db.profile.indicators[baseKey].duration = v
				Grid2Frame:UpdateAllFrames()
			end,
		},
		stack = {
			type = "toggle",
			name = L["Show stack"],
			desc = L["Show the number of stacks."],
			order = 80,
			tristate = true,
			get = function ()
				return indicator.dbx.stack
			end,
			set = function (_, v)
				indicator.dbx.stack = v
				Grid2.db.profile.indicators[baseKey].stack = v
				Grid2Frame:UpdateAllFrames()
			end,
		},
	}

	if Grid2Options.AddMediaOption then
		local fontOption = {
			type = "select",
			order = 70,
			name = L["Font"],
			desc = L["Adjust the font settings"],
		}
		Grid2Options:AddMediaOption("font", fontOption)
		local values = fontOption.values
		fontOption.get = function ()
			local fontIndex
			for index, handle in ipairs(values) do
				if (indicator.dbx.font == handle) then
					fontIndex = index
					break
				end
			end
			return fontIndex
		end
		fontOption.set = function (_, v)
			local fontHandle = values[v]
			indicator.dbx.font = fontHandle
			Grid2.db.profile.indicators[baseKey].font = fontHandle
			local font = media:Fetch("font", fontHandle)
			local fontsize = indicator.dbx.fontSize
			Grid2Frame:WithAllFrames(function (f) indicator:SetTextFont(f, font, fontsize) end)
		end
		options.font = fontOption
	end
	Grid2Options:MakeIndicatorTypeOptions(indicator, options)
	Grid2Options:AddIndicatorLocationOptions(indicator, options)
	Grid2Options:AddIndicatorDeleteOptions(indicator, options)

	Grid2Options:AddIndicatorStatusOptions(indicator, statuses)

	Grid2Options:AddIndicatorElement(indicator, options, statuses)

	local TextColor = Grid2.indicators[indicator.name .. "-color"]
	if TextColor then	
		options = {}
		Grid2Options:AddIndicatorStatusOptions(TextColor, options)
		Grid2Options:AddIndicatorElement(TextColor, options)	
	end
end

local function AddAlphaIndicatorOptions(indicator)
	local options = {}
	Grid2Options:AddIndicatorStatusOptions(indicator, options)
	Grid2Options:AddIndicatorElement(indicator, options)
end

local function MakeBarIndicatorOptions(indicator)
	local baseKey = indicator.name
	local options = {}
	local statuses= {}

	options.orientation = {
		type = "select",
		order = 10,
		name = L["Orientation of the Bar"],
		desc = L["Set status bar orientation."],
		get = function ()
			return indicator.dbx.orientation or "DEFAULT"
		end,
		set = function (_, v)
			if v=="DEFAULT" then v= nil	end
			indicator.orientation= v
			indicator.dbx.orientation = v
			Grid2.db.profile.indicators[baseKey].orientation = v	
			Grid2Frame:WithAllFrames(function (f) indicator:SetOrientation(f,v) end)
		end,
		values={ ["DEFAULT"]= L["DEFAULT"], ["VERTICAL"] = L["VERTICAL"], ["HORIZONTAL"] = L["HORIZONTAL"]}
	}
	options.frameLevel = {
		type = "select",
		order = 25,
		name = L["Frame Level"],
		desc = L["Bars with higher numbers always show up on top of lower numbers."],
		get = function ()
			return indicator.dbx.level or 1
		end,
		set = function (_, v)
			indicator.frameLevel = v
			indicator.dbx.level = v
			Grid2.db.profile.indicators[baseKey].level = v	
			Grid2Frame:WithAllFrames(function (f) indicator:Layout(f) end)
		end,
		values={ 1,2,3,4,5}
	}
	
	options.barWidth= {
		type = "range",
		order = 30,
		name = L["Bar Width"],
		desc = L["Choose zero to set the bar to the same width as parent frame"],
		min = 0,
		max = 75,
		step = 1,
		get = function ()
			return indicator.dbx.width
		end,
		set = function (_, v)
			if v==0 then v= nil end
			indicator.dbx.width = v
			indicator.width= v
			Grid2.db.profile.indicators[baseKey].width = v	
			Grid2Frame:WithAllFrames(function (f) indicator:Layout(f) end)
		end,	
	
	}
	options.barHeight= {
		type = "range",
		order = 40,
		name = L["Bar Height"],
		desc = L["Choose zero to set the bar to the same height as parent frame"],
		min = 0,
		max = 75,
		step = 1,
		get = function ()
			return indicator.dbx.height
		end,
		set = function (_, v)
			if v==0 then v= nil end
			indicator.dbx.height = v
			indicator.height= v
			Grid2.db.profile.indicators[baseKey].height = v	
			Grid2Frame:WithAllFrames(function (f) indicator:Layout(f) end)
		end,	
	
	}
	
	if Grid2Options.AddMediaOption then
		local textureOption = {
			type = "select",
			order = 20,
			name = L["Frame Texture"],
			desc = L["Adjust the texture of the bar."],
			get = function (info)
				local v = indicator.dbx.texture
				for i, t in ipairs(info.option.values) do
					if v == t then return i end
				end
			end,
			set = function (info, v)
				v = info.option.values[v]
				indicator.dbx.texture = v
				Grid2.db.profile.indicators[baseKey].texture = v
				local texture = media:Fetch("statusbar", v)
				Grid2Frame:WithAllFrames(function (f) indicator:SetTexture(f, texture) end)
			end,
		}
		Grid2Options:AddMediaOption("statusbar", textureOption)
		options.texture = textureOption
	end
	Grid2Options:AddIndicatorLocationOptions(indicator, options)
	Grid2Options:AddIndicatorDeleteOptions(indicator, options)

	Grid2Options:AddIndicatorStatusOptions(indicator, statuses)

	Grid2Options:AddIndicatorElement(indicator, options, statuses)
	
	local TextColor = Grid2.indicators[indicator.name .. "-color"]
	if TextColor then
		local funcMakeOptions = Grid2Options.typeMakeOptions["bar-color"]
		funcMakeOptions(TextColor)
	end	
end

local function AddBorderIndicatorOptions(indicator)
	local options = {}
	Grid2Options:AddIndicatorStatusOptions(indicator, options)
	Grid2Options:AddIndicatorElement(indicator, options)
end

local function MakeIconIndicatorOptions(indicator)
	local statuses= {}
	local options = {
		reverseCooldown = {
			type = "toggle",
			order = 90,
			name = L["Reverse Cooldown"],
			desc = L["Set cooldown to become darker over time instead of lighter."],
			tristate = true,
			get = function ()
				return indicator.dbx.reverseCooldown
			end,
			set = function (_, v)
				indicator.dbx.reverseCooldown = v
				
				--Apply changes to the bar dbx
				local indicatorKey = indicator.name
				Grid2.db.profile.indicators[indicatorKey].reverseCooldown = v

				Grid2Frame:WithAllFrames(function (f)
					f[indicatorKey].Cooldown:SetReverse(indicator.dbx.reverseCooldown)
				end)
			end,
		}
	}
	Grid2Options:MakeIndicatorSizeOptions(indicator, options)
	Grid2Options:MakeIndicatorTypeOptions(indicator, options)
	Grid2Options:AddIndicatorLocationOptions(indicator, options)
	Grid2Options:MakeIndicatorBorderOptions(indicator, options)
	Grid2Options:AddIndicatorDeleteOptions(indicator, options)
	Grid2Options:AddIndicatorStatusOptions(indicator, statuses)

	Grid2Options:AddIndicatorElement(indicator, options, statuses)
end

local function MakeSquareIndicatorOptions(indicator)
	local layout={}
	local statuses={}
	Grid2Options:MakeIndicatorSizeOptions(indicator, layout)
	Grid2Options:MakeIndicatorTypeOptions(indicator, layout)
	Grid2Options:AddIndicatorLocationOptions(indicator, layout)
	Grid2Options:MakeIndicatorBorderOptions(indicator, layout)
	Grid2Options:AddIndicatorDeleteOptions(indicator, layout)
	Grid2Options:AddIndicatorStatusOptions(indicator, statuses)	
	Grid2Options:AddIndicatorElement(indicator, layout, statuses)
end

local function MakeBarColorIndicatorOptions(indicator)
	local baseKey = indicator.name
	local options = {}
	local statuses= {}
	
	options.inverColor= {
		type = "toggle",
		name = L["Invert Bar Color"],
		desc = L["Swap foreground/background colors on bars."],
		order =10,
		tristate = true,
		get = function ()
			return indicator.dbx.invertColor
		end,
		set = function (_, v)
			indicator.dbx.invertColor = v
			Grid2.db.profile.indicators[baseKey].invertColor= v
			indicator:UpdateDB()
			if not v then
			    local c= Grid2Frame.db.profile.frameColor or {r=0,g=0,b=0,a=1}
				Grid2Frame:WithAllFrames(function (f) f:SetBackgroundColor(c.r, c.g, c.b, c.a) end)
			end	
			Grid2Frame:UpdateAllFrames()
			Grid2Frame:Reset()
		end,
	}	
	
	Grid2Options:AddIndicatorStatusOptions(indicator, statuses)
	Grid2Options:AddIndicatorElement(indicator, options, statuses)
end

-- 

local newIndicatorValues= { name="", type= "square", relPoint= "TOPLEFT" }

local function NewIndicator()
	local newIndicatorName = Grid2Options:GetValidatedName(newIndicatorValues.name)
	if (newIndicatorName and newIndicatorName ~= "") then
		-- save indicator in database
		local defaults= Grid2Options.typeDefaultValues
		local dbx= { type= newIndicatorValues.type }
		dbx.location= Grid2.CreateLocation(newIndicatorValues.relPoint)
		if (newIndicatorValues.type == "square") then
			dbx.level = 9
			dbx.size = defaults.square.size
		elseif (newIndicatorValues.type == "icon") then
			dbx.level = 8
			dbx.size = defaults.icon.size
			dbx.fontSize= defaults.icon.fontSize
		elseif (newIndicatorValues.type == "text") then
			dbx.level = 6
			dbx.textlength= defaults.text.textlength
			dbx.fontSize= defaults.text.fontSize
			dbx.font= defaults.text.font
			Grid2:DbSetIndicator( newIndicatorName.."-color" , { type="text-color" })
		elseif (newIndicatorValues.type == "bar") then
			dbx.level = 3
			dbx.texture= "Gradient"
			Grid2:DbSetIndicator( newIndicatorName.."-color" , { type="text-color" })
		end
		Grid2:DbSetIndicator(newIndicatorName,dbx)
		-- Create runtime indicator 
		local setupFunc = Grid2.setupFunc[dbx.type]
		local indicator = setupFunc(newIndicatorName, dbx)
		Grid2Frame:WithAllFrames(function (f)
			indicator:Create(f)
			indicator:Layout(f)
		end)
		-- Create indicator options
		local funcMakeOptions = Grid2Options.typeMakeOptions[dbx.type]
		if (funcMakeOptions) then
			funcMakeOptions(indicator)
		end
	end
end

local function NewIndicatorDisabled()
	local name = Grid2Options:GetValidatedName(newIndicatorValues.name)
	if (name and name ~= "") then
		if (not Grid2.indicators[name]) then
			return false
		end
	end
	return true
end

local function AddIndicatorsGroup(reset)
	local options = {
		newIndicatorName = {
			type = "input",
			order = 2,
			width = "full",
			name = L["Name"],
			desc = L["Name of the new indicator"],
			usage = L["<CharacterOnlyString>"],
			get = function()  return newIndicatorValues.name end,
			set = function(_,v)	newIndicatorValues.name= v  end,
		},
		newIndicatorType = {
			type = 'select',
			order = 3,
			name = L["Type"],
			desc = L["Type of indicator to create"],
			values = Grid2Options.GetNewIndicatorTypes,
			get = function() return newIndicatorValues.type end,
			set = function(_,v)	
				newIndicatorValues.type= v  
				if v=="icon" or v=="text" then
					newIndicatorValues.relPoint= "CENTER"
				elseif v=="bar" then
					newIndicatorValues.relPoint= "BOTTOM"
				else
					newIndicatorValues.relPoint= "TOPLEFT"
				end
			end,
		},
		newIndicatorLocation= {
		    type = 'select',
			order = 4,
			name = L["Location"],
			desc = L["Align my align point relative to"],
		    values = pointValueList,
			get = function() return pointMap[newIndicatorValues.relPoint] end,
			set = function(_, v) newIndicatorValues.relPoint= pointMap[v] end,
		},
		newIndicator = {
			type = "execute",
			order = 9,
			name = L["New Indicator"],
			desc = L["Create a new indicator."],
			func = NewIndicator,
			disabled = NewIndicatorDisabled,
		},
	   resetIndicatorsSpacer = {
			type = "header",
			order = 12,
			name = "",
	   },
}

	Grid2Options:AddElementGroup("indicators", options, 60, reset)
	
	--Grid2Options.options.args["indicators"] = options
end


local indicatorTypesOrder= { 
	["alpha"] = 10,
	["border"] = 11,
	["bar"] = 12,
	["square"] = 50,
	["text"] = 60,
	["icon"] = 70,
}

--/dump Grid2Options.options.Grid2.args.indicator
function Grid2Options:AddIndicatorElement(element, layoutOptions, statusOptions)
	local insertLayout
	local insertStatus
	local type= string.gsub(element.dbx.type, "-color", "")
	local options = {}
	local insertPoint =  self.options.args.indicators

	-- indicator name language translation	
	local name= string.gsub(element.name, "-color", "")
	local lname= L[name]
	if lname==name then
		lname= string.gsub(lname,"-"," ")
	end
	if name ~= element.name then
		lname= lname .. L["-color"]
	end
	
	insertPoint.args[element.name] = {
		type = "group",
		childGroups= "tab",
		order= indicatorTypesOrder[type] or 100,
		name = lname,
		desc = L["Options for %s."]:format(name),
		args = options,
	}
	
	if statusOptions then
		insertLayout= {}
		insertStatus= {}
		options["layout"]=  {
			type="group",
			order= 20,
			name = L["Layout"],
			args= insertLayout,
		}
		options["statuses"]= {
			type="group",
			order= 10,
			name = L["statuses"],
			args= insertStatus,
		}
		for name, option in pairs(statusOptions) do
			insertStatus[name] = option
		end
	else
		insertLayout= options
	end
	
	for name, option in pairs(layoutOptions) do
		insertLayout[name] = option
	end
	
end

function Grid2Options:DeleteIndicatorElement(objectKey)
	local insertionPoint = self.options.args.indicators
	insertionPoint.args[objectKey] = nil
end

--No options for the indicator
function Grid2Options:MakeNoIndicatorOptions()
end

function Grid2Options:AddCreatableOptionHandler(typeKey, name, funcMakeOptions, optionParams)
	newIndicatorTypes[typeKey] = name
	self:AddOptionHandler(typeKey, funcMakeOptions, optionParams)
end

function Grid2Options:MakeIndicatorOptions(reset)
	AddIndicatorsGroup(reset)

	self:AddOptionHandler("alpha", AddAlphaIndicatorOptions)
	self:AddOptionHandler("bar", MakeBarIndicatorOptions)
	self:AddOptionHandler("bar-color", MakeBarColorIndicatorOptions)
	self:AddOptionHandler("border", AddBorderIndicatorOptions)

	self:AddCreatableOptionHandler("icon", L["icon"], MakeIconIndicatorOptions)
	self:AddCreatableOptionHandler("square", L["square"], MakeSquareIndicatorOptions)
	self:AddCreatableOptionHandler("text", L["text"], MakeTextIndicatorOptions)
	self:AddCreatableOptionHandler("bar", L["bar"], MakeBarIndicatorOptions)
	self:AddOptionHandler("text-color", Grid2Options.MakeNoIndicatorOptions)
 
    local indicators= Grid2.db.profile.indicators
 
	for baseKey, dbx in pairs(indicators) do
		local indicator = Grid2.indicators[baseKey]
		if (dbx) then
			local funcMakeOptions = Grid2Options.typeMakeOptions[dbx.type]
			if (funcMakeOptions) then
				funcMakeOptions(indicator)
			else
				print("    **MakeIndicatorOptions no funcMakeOptions for", baseKey, dbx.type)
			end
		else
			print("    **MakeIndicatorOptions no dbx for", baseKey)
		end
	end
end

